from typing import List
from typing import Optional
from typing import Union

from cleaninsights.campaign import Campaign
from cleaninsights.conf import Configuration
from cleaninsights.store import Store


class CleanInsights:
    conf: Configuration
    store: Store
    persistence_counter: int

    def __init__(self, conf, store):
        self.conf = conf
        self.store = store

    def measure_visit(self, path: str, campaign_id: str):
        raise NotImplementedError

    def measure_event(self, category: str,
                      action: str,
                      campaign_id: str,
                      name: Optional[str] = None,
                      value: Union[int, float] = None):
        raise NotImplementedError

    @property
    def feature_consents(self) -> List[str]:
        raise NotImplementedError

    @property
    def campaign_consents(self) -> List[str]:
        raise NotImplementedError

    def feature_consent_by_index(self, index: int):
        raise NotImplementedError

    def campaign_consent_by_index(self, index: int):
        raise NotImplementedError

    def grant_feature(self, feature):
        raise NotImplementedError

    def deny_feature(self, feature):
        raise NotImplementedError

    def grant_campaign(self, campaign_id: str):
        raise NotImplementedError

    def deny_campaign(self, campaign_id: str):
        raise NotImplementedError

    def is_campaign_currently_granted(self, campaign_id: str) -> bool:
        raise NotImplementedError

    def persist(self) -> None:
        raise NotImplementedError

    def get_campaign_if_good(self, campaign_id: str,
                             debug_str: str) -> Optional[Campaign]:
        raise NotImplementedError



from dataclasses import dataclass
from datetime import date
from typing import Any
from typing import Dict
from typing import Optional


@dataclass
class Campaign:
    start: date
    end: date
    aggregation_period_length: int
    number_of_periods: Optional[int]
    only_record_once: bool
    event_aggregation_rule: str

    @classmethod
    def from_dict(cls, campaign_data: Dict[str, Any]):  # -> Campaign
        raise NotImplementedError



from datetime import datetime
from typing import Optional


class DataPoint:
    campaign_id: str
    times: int
    first: datetime
    last: datetime

    def __init__(self, campaign_id: str, times: Optional[int],
                 first: Optional[datetime], last: Optional[datetime]):
        self.campaign_id = campaign_id
        self.times = 1 if times is None else times
        self.first = datetime.utcnow() if first is None else first
        self.last = datetime.utcnow() if last is None else last



from datetime import datetime
from typing import Optional
from typing import Union

from cleaninsights.datapoint import DataPoint


class Event(DataPoint):
    category: str
    action: Optional[str]
    name: Optional[str]
    value: Optional[Union[int, float]]

    def __init__(self, category: str, action: Optional[str],
                 name: Optional[str], value: Optional[Union[int, float]],
                 campaign_id: str, times: Optional[int],
                 first: Optional[datetime], last: Optional[datetime]):
        super().__init__(campaign_id, times, first, last)
        self.category = category
        self.action = action
        self.name = name
        self.value = value



from dataclasses import dataclass
from typing import Any
from typing import Dict

from cleaninsights.campaign import Campaign


@dataclass
class Configuration:
    server: str
    site_id: int
    campaigns: Dict[str, Campaign]
    timeout: int
    persist_every_n_times: int
    server_side_anon_usage: bool
    debug: bool

    @classmethod
    def from_dict(cls, conf_data: Dict[str, Any]):  # -> Configuration
        raise NotImplementedError



from datetime import datetime
from typing import List
from typing import Optional

from cleaninsights.datapoint import DataPoint


class Visit(DataPoint):
    path: List[str]

    def __init__(self, path: List[str], campaign_id: str, times: Optional[int],
                 first: Optional[datetime], last: Optional[datetime]):
        super().__init__(campaign_id, times, first, last)
        self.path = path



from typing import Dict
from typing import List
from typing import Optional
from typing import Union

from cleaninsights.event import Event
from cleaninsights.visit import Visit


class Store:
    events: List[Event]
    visits: List[Visit]

    def __init__(self, *args, **kwargs):
        data = self.load(*args, **kwargs)
        if data is None:
            raise RuntimeError("Storage doesn't exist or isn't readable.")
        if 'events' in data:
            self.events = [Event(**e) for e in data['events']]
        if 'visits' in data:
            self.visits = [Visit(**v) for v in data['visits']]

    def load(self, *args,
             **kwargs) -> Optional[List[Dict[str, Union[str, int, float]]]]:
        raise NotImplementedError

    def persist(self) -> None:
        raise NotImplementedError

    def send(self, data: str, server: str, timeout: int) -> bool:
        raise NotImplementedError



from os import path

from setuptools import find_packages
from setuptools import setup

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

with open(path.join(here, 'requirements.txt'), encoding='utf-8') as f:
    install_requires = f.read().splitlines()

setup(
    name="cleaninsights",
    version="0.0",
    packages=find_packages(exclude=['docs']),
    install_requires=install_requires,
    author="Iain Learmonth / Guardian Project",
    author_email="iain@learmonth.me",
    description=("CleanInsights gives developers a safe, sustainable, and "
                 "secure way to gather insights about their users using "
                 "cutting edge techniques like differential privacy, onion "
                 "routing, certificate pinning and more."),
    long_description=long_description,
    keywords="privacy metrics analytics",
    url="https://gitlab.com/cleaninsights/clean-insights-python-sdk/",
    project_urls={
        "Bug Tracker":
        "https://gitlab.com/keanuapp/clean-insights-python-sdk/-/issues",
        "Source Code": "https://gitlab.com/keanuapp/clean-insights-python-sdk",
    },
    classifiers=["License :: OSI Approved :: BSD License"],
    test_suite='nose.collector',
)
